************************************************************************
*
* PCG: Preconditioned Conjugate Gradient Package
* Version: f77
*
************************************************************************
************************************************************************
*# 
*#                      COPYRIGHT/DISCLAIMER NOTICE
*#
*# This program was prepared by the Regents of the University of
*# California at Los Alamos National Laboratory under Contract No.
*# W-7405-ENG-36 with the U.S. Department of Energy (DOE), and by
*# the University of Texas at Austin under ARPA Contract No.
*# DABT63-92-C-0024.
*# 
*# The University of California and the University of Texas at
*# Austin have certain rights in the program pursuant to these
*# contracts.
*# 
*# Permission is hereby granted to use the program for the user's
*# own internal use.  The user is not granted the right to reproduce,
*# prepare derivative works, or redistribute the program without
*# prior permission of the University of California or the University
*# of Texas at Austin.
*# 
*# All rights in the program are reserved by the University of
*# California and the University of Texas at Austin.
*# 
*# Portions of this material resulted from work developed under a
*# U.S. Government Contract and are subject to the following
*# license: the Government is granted for itself and others acting
*# on its behalf a paid-up, nonexclusive, irrevocable worldwide
*# license in this computer software to reproduce, prepare derivative
*# works, and perform publicly and display publicly.
*# 
*# Neither the U.S. Government, the University of California nor
*# the University of Texas at Austin, nor any of their employees,
*# makes any warranty, express or implied, or assumes any liability
*# or responsibility for the use of this software.
*# 
*# Copyright (c) 1992-1995 the University of California and the
*# University of Texas at Austin.
*# 
************************************************************************
************************************************************************
************************************************************************
**  ^ROUTINE: WGMRRR - Reverse communication level routine for
**                     restarted GMRESRacceleration. (^)
**
************************************************************************
**
      subroutine dgmrrr ( ijob , ireq ,
     &       u , uexact , b ,
     &       iva , ivql , ivqr ,
     &       iwk , fwk , iparm , fparm , ier )
**
************************************************************************
**
**  ^DESCRIPTION:
**    Restarted GMRESR iterative method.  This method basically
**    accelerates GMRES by means of an outer GCR acceleration.
**    This is a two-level scheme, using ns2 as the restart frequency
**    for the inner level and ns1 for the outer.
**
**  ^AUTHOR:   wdj@beta.lanl.gov
**
**  ^MODIFIED: wdj@lithos.c3.lanl.gov Thu May  6 13:12:38 MDT 1993
**
**  ^ARGUMENTS: see *Subroutine Arguments* below.
**
**  ^REQUIREMENTS:
**    Common Blocks: none
**    Subroutines:   see below.
**
**  ^SIDE_EFFECTS:
**    See the definition of METHRDCLS to see which parameters
**    are modified
**
**  ^ALGORITHM:
**
**  ^REFERENCES:
**#   H. A. Van der Vorst and C. Vuik, ``GMRESR: a family of nested
**    GMRES methods,'' {\sl Numerical Linear Algebra with Applications},
**    Vol. 1 (1), pp. 1--20 (1993)a.
**
**  ^DOCUMENTATION:
**
**  ^MACROS:
**
************************************************************************
*
*     implicit character*1 (a-z)
*
*#    METHRAL        - (^)
      integer ijob
      integer ireq
      integer ier
      integer iwk(*)
      integer iparm(*)
      double precision       fwk(*)
      double precision       fparm(*)
      double precision u(*)
      double precision uexact(*)
      double precision b(*)
      integer ivql
      integer iva
      integer ivqr
*
        integer itimer
        double precision tc1, tr1, tc2, tr2
      integer    iitab
      integer    iftab
      integer    iwkstg
      integer    iclstg
      integer    in
      integer    is
      integer    iretlb
      integer    ivitmp, ivotmp
      integer    iv1tst, iv2tst
      integer    istab1, istab2, istab
      integer    ib1,ib2, iu1,iu2
      integer    iud
      parameter (iud  =0)
      integer    ibd
      parameter (ibd  =1)
      integer    ilbd
      parameter (ilbd =2)
      integer    irlbd
      parameter (irlbd=3)
      integer    ied
      parameter (ied  =4)
      integer    ird
      parameter (ird  =5)
      integer    ilrd
      parameter (ilrd =6)
      integer    irlrd
      parameter (irlrd=7)
      double precision      urndf
      double precision      top
      double precision      bot
      parameter (urndf=.1)
      double precision      mydot1, mydot2
      external   dargck
      external   dckstg
      external   dwrnd
      external   difppr
      external   ditput
      external   xtimer
        integer ipme
        integer nproc
      external ximini
      external ximal
      external xifre
      external dfmini
      external dfmal
      external dffre
*     integer imv1, imv2
      integer    ir
      integer    ilr
      integer    irlr
      integer    ip
      integer    ilp
      integer    irlp
      integer    irlq
      integer    iarlq
      integer    ilarlq
      integer    irarlq
      integer    iv1
      integer    iv2
      integer    ipf, icos, isin, iu, izc, iy, iyold, ipz, ipzold
      integer    ns, nsout
      integer    i, j
      integer    os
      integer    nm
      double precision      rdot
      double precision      rnorm
      double precision      r0norm
      double precision      uudot, uexdt
      double precision      vnorm, pap, vdot, ut, h, v1, v2, denom, sum,
     & val
*
*
****^^******************************************************************
*     $Modified: wdj@lanl.gov Mon Aug 28 19:20:55 MDT 1995
*     $Id: gmrrr.fm4,v 1.14 1994/11/22 05:20:25 joubert Exp $
*     $Revision: 1.0 $
*     $Log: gmrrr.fm4,v $
************************************************************************
*
      iitab = (iparm(6))
      iftab = (iparm(6))
      ir = (iparm(6))
      ilr = (iparm(6))
      irlr = (iparm(6))
      ip = (iparm(6))
      ilp = (iparm(6))
      irlp = (iparm(6))
      irlq = (iparm(6))
      iarlq = (iparm(6))
      ilarlq = (iparm(6))
      irarlq = (iparm(6))
      iv1 = (iparm(6))
      iv2 = (iparm(6))
      ipf = (iparm(6))
      icos = (iparm(6))
      isin = (iparm(6))
      iu = (iparm(6))
      izc = (iparm(6))
      iy = (iparm(6))
      iyold = (iparm(6))
      ipz = (iparm(6))
      ipzold = (iparm(6))
*
*     *---go to the brancher unless this is the init call---
      if (ijob .ne. 1) go to 1000
*
*     *---inits----------
      iclstg = 1
      itimer = 0
      in = 0
      iparm(5) = in
      is = 0
*
*     *---initialize pointers----------
      iu1 = (iparm(6))
      iu2 = (iparm(6))
      ib1 = (iparm(6))
      ib2 = (iparm(6))
      istab1 = (iparm(6))
      istab2 = (iparm(6))
*
*     *---initialize----------
      ipme  = 0
      nproc = 1
      if (iparm(27) .eq. 0) iparm(27) = 0
      if (iparm(27) .eq. 0) then
        ier = 0
        if (ijob.eq.1 .or. ijob.eq.2 .or. ijob.eq.0) then
*       *---init integer memory----------
        call ximini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*       *---init floating point memory----------
        call dfmini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
*       *---check arguments----------
        call dargck ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*       *---initialize timer----------
        call xtimer (tc1,tr1,iparm(22),1)
      endif
*
*     *---start timer------
      call xtimer (tc1,tr1,iparm(22),0)
      itimer = 1
*
*     *---print i/fparm if requested----------
      if (iparm(27) .eq. 0) then
        call difppr ( 1 , iparm, fparm, ier)
           if (ier .lt. 0) go to 900
      endif
*
*     *---reserve place to save integer and float variables----------
*     *---across revcom calls----------
      call ximal (iitab,46, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (iftab,5, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
      if (iparm(2) .ge. 3) then
        write (iparm(1),'(/''     Restarted GMRESR Method.''/)') 
      endif
*
      ns    = min(iparm(17),iparm(4))
      nsout = min(iparm(16),(iparm(4)+ns-1)/ns)
*
      call dfmal (ilr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (ilp,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (irlq,iparm(3)*(nsout), iparm , fparm , iwk , fwk , ier
     & )
           if (ier .lt. 0) go to 900
      call dfmal (ilarlq,iparm(3)*(nsout), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
      if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dfmal (ir,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          call dfmal (ip,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ie
     &r )
           if (ier .lt. 0) go to 900
          call dfmal (iarlq,iparm(3)*(nsout), iparm , fparm , iwk , fwk 
     &, ier )
           if (ier .lt. 0) go to 900
        endif
      else
        ir = ilr
        iarlq = ilarlq
      endif
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        call dfmal (irlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dfmal (irlp,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ie
     &r )
           if (ier .lt. 0) go to 900
        call dfmal (irarlq,iparm(3)*(nsout), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
      else
        irlr = ilr
        irlp = ilp
        irarlq = ilarlq
      endif
      call dfmal (iv1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (iv2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (ipf   ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (icos  ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (isin  ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (iu    ,ns*(ns+1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (izc   ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (iy    ,ns, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (iyold ,ns, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (ipz   ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call dfmal (ipzold,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
*
*     *---set u to zero----------
      if (iparm(14) .eq. -2
     &    .or. iparm(14) .eq. -1) then
          call dwfil (u,0d0,
     &     iparm , fparm , iwk , fwk , ier  )
      endif
*
*     *---random u----------
      if (iparm(14) .eq. 2
     &    ) go to 5500
                 go to 5501
 5500  continue
          call dwrdot (mydot1,b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
        call dwrnd (u,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv1 
         ireq = 3
        iretlb = 2000
        go to 1500
 2000   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwrdot (mydot2,fwk(iv1),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call dwscl ( u, u, urndf*sqrt(mydot1/mydot2),  iparm , fparm ,
     & iwk , fwk , ier  )
                go to 5502
 5501 continue
 5502 continue
*
*     *---random u perturbation----------
      if (iparm(14) .eq. 3) go to 5503
                 go to 5504
 5503  continue
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv1 
         ireq = 3
        iretlb = 2001
        go to 1500
 2001   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwsub ( fwk(iv1),
     &    b, fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call dwrdot (mydot1,fwk(iv1),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call dwset (fwk(iv1),u,
     &     iparm , fparm , iwk , fwk , ier  )
        call dwrnd (u,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv2 
         ireq = 3
        iretlb = 2002
        go to 1500
 2002   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwrdot (mydot2,fwk(iv2),
     &    fwk(iv2),  iparm , fparm , iwk , fwk , ier  )
          call dwsax (u,fwk(iv1),
     &    u, -urndf*sqrt(mydot1/mydot2),  iparm , fparm , iwk , fwk , ie
     &r  )
                go to 5505
 5504 continue
 5505 continue
*
*     *---begin iteration statistics calculation----------
      if (iparm(21) .eq. 1) go to 5506
                 go to 5507
 5506  continue
        call dfmal (istab1,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dfmal (istab2,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        istab = istab1
        if (iparm(19) .eq. 1) then
          call dwdot (fwk(istab+iud), uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
        endif
          call dwdot (fwk(istab+ibd ), b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5509
                 go to 5510
 5509  continue
          call dwset (fwk(ib2),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5511
 5510 continue
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call dwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = ib2 
         ireq = 5
        iretlb = 2003
        go to 1500
 2003   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5511 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+ilbd), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call dwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5512
                 go to 5513
 5512  continue
          call dwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5514
 5513 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2004
        go to 1500
 2004   continue
         if (ijob .eq. -1) go to 900
 5514 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+irlbd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(19) .eq. 1) then
          call dwsub ( fwk(iu1),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call dwdot (fwk(istab+ied), fwk(iu1),
     &    fwk(iu1),  iparm , fparm , iwk , fwk , ier  )
        endif
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ib1 
         ireq = 3
        iretlb = 2005
        go to 1500
 2005   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwsub ( fwk(ib1),
     &    b, fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
          call dwdot (fwk(istab+ird), fwk(ib1),
     &    fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5515
                 go to 5516
 5515  continue
          call dwset (fwk(ib2),fwk(ib1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5517
 5516 continue
          iva = ib1 
          ivql = ib2 
         ireq = 5
        iretlb = 2006
        go to 1500
 2006   continue
         if (ijob .eq. -1) go to 900
 5517 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+ilrd ), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call dwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5518
                 go to 5519
 5518  continue
          call dwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5520
 5519 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2007
        go to 1500
 2007   continue
         if (ijob .eq. -1) go to 900
 5520 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+irlrd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(2) .ge. 4) then
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        write (iparm(1),'(5x,''Initial iteration statistics'')') 
        write (iparm(1),'()') 
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10)'
     &) sqrt(fwk(istab+iud))
        endif
        write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10)'
     &) sqrt(fwk(istab+ibd))
        write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10)'
     &) sqrt(fwk(istab+ilbd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10)'
     &) sqrt(fwk(istab+irlbd))
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of error             '',1pg20.10)'
     &) sqrt(fwk(istab+ied))
        endif
        write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10)'
     &) sqrt(fwk(istab+ird))
        write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10)'
     &) sqrt(fwk(istab+ilrd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10)'
     &) sqrt(fwk(istab+irlrd))
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        endif
        call dffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call dffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
                go to 5508
 5507 continue
 5508 continue
*     *---end iteration statistics calculation----------
*
      if (((iparm(14).eq.0).or.(iparm(14).eq.-2)) .or. iparm(14).eq.-1) 
     &go to 5521
                 go to 5522
 5521  continue
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5524
                 go to 5525
 5524  continue
          if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) go to 5527
                 go to 5528
 5527  continue
          call dwset (fwk(ir),b,
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5530
                 go to 5531
 5530  continue
          call dwset (fwk(ilr),fwk(ir),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5532
 5531 continue
          iva = ir 
          ivql = ilr 
         ireq = 5
        iretlb = 2008
        go to 1500
 2008   continue
         if (ijob .eq. -1) go to 900
 5532 continue
*        *---end revcom precon----------
                go to 5529
 5528 continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5533
                 go to 5534
 5533  continue
          call dwset (fwk(ilr),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5535
 5534 continue
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call dwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = ilr 
         ireq = 5
        iretlb = 2009
        go to 1500
 2009   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5535 continue
*        *---end revcom precon----------
 5529 continue
                go to 5526
 5525 continue
          call dwset (fwk(ilr),b,
     &     iparm , fparm , iwk , fwk , ier  )
 5526 continue
                go to 5523
 5522 continue
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5536
                 go to 5537
 5536  continue
          if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) go to 5539
                 go to 5540
 5539  continue
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ir 
         ireq = 3
        iretlb = 2010
        go to 1500
 2010   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwsub ( fwk(ir),
     &    b, fwk(ir),  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5542
                 go to 5543
 5542  continue
          call dwset (fwk(ilr),fwk(ir),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5544
 5543 continue
          iva = ir 
          ivql = ilr 
         ireq = 5
        iretlb = 2011
        go to 1500
 2011   continue
         if (ijob .eq. -1) go to 900
 5544 continue
*        *---end revcom precon----------
                go to 5541
 5540 continue
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv1 
         ireq = 3
        iretlb = 2012
        go to 1500
 2012   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwsub ( fwk(iv1),
     &    b, fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5545
                 go to 5546
 5545  continue
          call dwset (fwk(ilr),fwk(iv1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5547
 5546 continue
          iva = iv1 
          ivql = ilr 
         ireq = 5
        iretlb = 2013
        go to 1500
 2013   continue
         if (ijob .eq. -1) go to 900
 5547 continue
*        *---end revcom precon----------
 5541 continue
                go to 5538
 5537 continue
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ilr 
         ireq = 3
        iretlb = 2014
        go to 1500
 2014   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwsub ( fwk(ilr),
     &    b, fwk(ir),  iparm , fparm , iwk , fwk , ier  )
 5538 continue
 5523 continue
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5548
                 go to 5549
 5548  continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5551
                 go to 5552
 5551  continue
          call dwset (fwk(irlr),fwk(ilr),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5553
 5552 continue
          ivql = ilr 
          ivqr = irlr 
         ireq = 7
        iretlb = 2015
        go to 1500
 2015   continue
         if (ijob .eq. -1) go to 900
 5553 continue
*        *---end revcom precon----------
                go to 5550
 5549 continue
 5550 continue
*
          call dwrdot (rdot,fwk(ilr),
     &    fwk(ilr),  iparm , fparm , iwk , fwk , ier  )
      rnorm  = sqrt(abs(rdot))
      r0norm = rnorm
      os = 0
*
*---------------------------iteration loop------------------------------
*
 100  continue
*
        if (iparm(2) .ge. 5) then
        write (iparm(1),500) 'QL r norm           ', rnorm
        endif
*
*     *---begin user stopping test----------
        if (iparm(12).eq.-3) go to 5554
                 go to 5555
 5554  continue
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          iva = ir 
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          ivql = ilr 
          endif
          if ((mod(iparm(15)/8,2).eq.1)) then
          ivqr = irlr 
          endif
          ireq   = 9
        iretlb = 2016
        go to 1500
 2016   continue
          if ((mod(iparm(15)/8,2).eq.1)) then
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          endif
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if (ier .ge. 0) then
            call ditput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          endif
          if (ijob .eq. -1) then
            if (iparm(2).ge.3 .and. iparm(27).eq.0) then
        write (iparm(1),'(/a/)') ' User-requested termination of iterati
     &ve method.'
            endif
            go to 900
          endif
                go to 5556
 5555 continue
 5556 continue
        if (iparm(12) .gt. 1) go to 5557
                 go to 5558
 5557  continue
          if (iparm(12) .eq. 1 ) then
            if (iparm(5) .eq. 0) then
              if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        '19 and 12 incompatible' )
                go to 900
              endif
          call dwdot (bot, uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
            endif
            call dfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier
     & )
           if (ier .lt. 0) go to 900
          call dwsub ( fwk(iv1tst),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call dwdot (top, fwk(iv1tst),
     &    fwk(iv1tst),  iparm , fparm , iwk , fwk , ier  )
            call dffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
          endif
          if (iparm(12) .eq. 2 ) then
            if (iparm(5) .eq. 0) then
          call dwdot (bot, b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
            endif
          call dwdot (top, fwk(ir),
     &    fwk(ir),  iparm , fparm , iwk , fwk , ier  )
          endif
          if (iparm(12) .eq. 3) go to 5560
                 go to 5561
 5560  continue
            if (iparm(5) .eq. 0) go to 5563
                 go to 5564
 5563  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5566
                 go to 5567
 5566  continue
          call dwdot (bot, fwk(ilr),
     &    fwk(ilr),  iparm , fparm , iwk , fwk , ier  )
                go to 5568
 5567 continue
                call dfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5569
                 go to 5570
 5569  continue
          call dwset (fwk(iv1tst),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5571
 5570 continue
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call dwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = iv1tst 
         ireq = 5
        iretlb = 2017
        go to 1500
 2017   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5571 continue
*        *---end revcom precon----------
          call dwdot (bot, fwk(iv1tst),
     &    fwk(iv1tst),  iparm , fparm , iwk , fwk , ier  )
                call dffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , f
     &wk , ier )
           if (ier .lt. 0) go to 900
 5568 continue
                go to 5565
 5564 continue
 5565 continue
          call dwdot (top, fwk(ilr),
     &    fwk(ilr),  iparm , fparm , iwk , fwk , ier  )
                go to 5562
 5561 continue
 5562 continue
          if (iparm(12) .eq. 4) go to 5572
                 go to 5573
 5572  continue
            if (iparm(5) .eq. 0) go to 5575
                 go to 5576
 5575  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5578
                 go to 5579
 5578  continue
          call dwdot (bot, fwk(irlr),
     &    fwk(irlr),  iparm , fparm , iwk , fwk , ier  )
                go to 5580
 5579 continue
                call dfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
                call dfmal (iv2tst,iparm(3), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5581
                 go to 5582
 5581  continue
          call dwset (fwk(iv1tst),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5583
 5582 continue
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call dwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = iv1tst 
         ireq = 5
        iretlb = 2018
        go to 1500
 2018   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5583 continue
*        *---end revcom precon----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5584
                 go to 5585
 5584  continue
          call dwset (fwk(iv2tst),fwk(iv1tst),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5586
 5585 continue
          ivql = iv1tst 
          ivqr = iv2tst 
         ireq = 7
        iretlb = 2019
        go to 1500
 2019   continue
         if (ijob .eq. -1) go to 900
 5586 continue
*        *---end revcom precon----------
          call dwdot (bot, fwk(iv2tst),
     &    fwk(iv2tst),  iparm , fparm , iwk , fwk , ier  )
                call dffre (iv2tst,iparm(3)*(1), iparm , fparm , iwk , f
     &wk , ier )
           if (ier .lt. 0) go to 900
                call dffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , f
     &wk , ier )
           if (ier .lt. 0) go to 900
 5580 continue
                go to 5577
 5576 continue
 5577 continue
          call dwdot (top, fwk(irlr),
     &    fwk(irlr),  iparm , fparm , iwk , fwk , ier  )
                go to 5574
 5573 continue
 5574 continue
          if (iparm(12) .eq. 1 ) then
            if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        '19 and 12 incompatible' )
                go to 900
            endif
            if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) then
          call dwdot (top, uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
            else
              call dfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
          call dwsub ( fwk(iv1tst),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call dwdot (top, fwk(iv1tst),
     &    fwk(iv1tst),  iparm , fparm , iwk , fwk , ier  )
              call dffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk
     & , ier )
           if (ier .lt. 0) go to 900
            endif
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 6) then
          call dwdot (top, fwk(ir),
     &    fwk(ir),  iparm , fparm , iwk , fwk , ier  )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 7) then
          call dwdot (top, fwk(ilr),
     &    fwk(ilr),  iparm , fparm , iwk , fwk , ier  )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 8) then
          call dwdot (top, fwk(irlr),
     &    fwk(irlr),  iparm , fparm , iwk , fwk , ier  )
            if (iparm(5) .eq. 0) bot = top
          endif
          if ((top) .lt. (0d0)) then
                ier = -10
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Attempt to take sqrt of negative number' 
     &)
                go to 900
          endif
          if ((bot) .le. (0d0)) then
                ier = -10
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Attempt to divide by zero' )
                go to 900
          endif
          fparm(5) = sqrt((top)/(bot))
          call ditput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          if ((fparm(5)) .le. (fparm(4))) go to 2500
                go to 5559
 5558 continue
 5559 continue
*     *---end user stopping test----------
*
        if (iparm(12) .eq. -1) then
          if (r0norm .eq. 0d0) then
                ier = -10
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Attempt to divide by zero' )
                go to 900
          endif
          fparm(5) = rnorm/r0norm
          call ditput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          if ((fparm(5)) .le. (fparm(4))) go to 2500
        endif
*
*       *---begin exact stopping test----------
        if (iparm(12) .eq. -2) then
          call dwsub ( fwk(iv1),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call dwrdot (uudot,fwk(iv1),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call dwrdot (uexdt,uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
          if ((uudot)  .lt. (0d0)) then
                ier = -10
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Attempt to take sqrt of negative number' 
     &)
                go to 900
          endif
          if ((uexdt) .le. (0d0)) then
                ier = -10
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Attempt to divide by zero' )
                go to 900
          endif
          fparm(5) = sqrt((uudot)/(uexdt))
          call ditput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          if ((fparm(5)) .le. (fparm(4))) go to 2500
        endif
*       *---end exact stopping test----------
*
        if (in .ge. iparm(4)) then
                ier = 2
                call xersho ( ier, 'dgmrrr' , iparm , ' ' )
          go to 900
        endif
*
        call dckstg ( iclstg, iwkstg,  iparm , fparm , iwk , fwk , ier  
     &)
           if (ier .lt. 0) go to 900
*
*-----------------------proceed with iteration--------------------------
*
        if (is .eq. 0) then
          call dwset (fwk(ilp+iparm(3)*((mod((is),ns+1)))),fwk(ilr),
     &     iparm , fparm , iwk , fwk , ier  )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(1
     &5)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dwset (fwk(ip+iparm(3)*((mod((is),ns+1)))),fwk(ir),
     &     iparm , fparm , iwk , fwk , ier  )
          endif
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call dwset (fwk(irlp+iparm(3)*((mod((is),ns+1)))),fwk(irlr),
     &     iparm , fparm , iwk , fwk , ier  )
          endif
          vnorm = rnorm
          if (vnorm .eq. 0d0) then
                ier = -7
                call xersho ( ier, 'dgmrrr' , iparm , ' ' )
                go to 900
          endif
          fwk(ipf    + mod((is),ns+1))   = 1d0/vnorm
          fwk(izc    + mod((is+1)-1,ns+1)) = vnorm
          fwk(ipz    + mod((1)-1,ns+1))    = 0d0
          if (iparm(2) .ge. 5) then
        write (iparm(1),500) 'h(1,0)              ', vnorm
          endif
        endif
*
        go to (501,502,503,504), iparm(13)+1
 501    continue
*        *---begin revcom matvec----------
          ivqr = ilp + iparm(3)*((mod((is),ns+1)))
          iva = ilp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 3
        iretlb = 2020
        go to 1500
 2020   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
        go to 510
 502    continue
          if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) go to 5587
                 go to 5588
 5587  continue
*        *---begin revcom matvec----------
          ivqr = ilp + iparm(3)*((mod((is),ns+1)))
          iva = ip + iparm(3)*((mod((is+1),ns+1)))
         ireq = 3
        iretlb = 2021
        go to 1500
 2021   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5590
                 go to 5591
 5590  continue
          call dwset (fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),fwk(ip+ipar
     &m(3)*((mod((is+1),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5592
 5591 continue
          iva = ip + iparm(3)*((mod((is+1),ns+1)))
          ivql = ilp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 5
        iretlb = 2022
        go to 1500
 2022   continue
         if (ijob .eq. -1) go to 900
 5592 continue
*        *---end revcom precon----------
                go to 5589
 5588 continue
*        *---begin revcom matvec----------
          ivqr = ilp + iparm(3)*((mod((is),ns+1)))
          iva = iv1  
         ireq = 3
        iretlb = 2023
        go to 1500
 2023   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5593
                 go to 5594
 5593  continue
          call dwset (fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),fwk(iv1 ),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5595
 5594 continue
          iva = iv1  
          ivql = ilp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 5
        iretlb = 2024
        go to 1500
 2024   continue
         if (ijob .eq. -1) go to 900
 5595 continue
*        *---end revcom precon----------
 5589 continue
        go to 510
 503    continue
*        *---begin revcom matvec----------
          ivqr = irlp + iparm(3)*((mod((is),ns+1)))
          iva = ilp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 3
        iretlb = 2025
        go to 1500
 2025   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5596
                 go to 5597
 5596  continue
          call dwset (fwk(irlp+iparm(3)*((mod((is+1),ns+1)))),fwk(ilp+ip
     &arm(3)*((mod((is+1),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5598
 5597 continue
          ivql = ilp + iparm(3)*((mod((is+1),ns+1)))
          ivqr = irlp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 7
        iretlb = 2026
        go to 1500
 2026   continue
         if (ijob .eq. -1) go to 900
 5598 continue
*        *---end revcom precon----------
        go to 510
 504    continue
          if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) go to 5599
                 go to 5600
 5599  continue
*        *---begin revcom matvec----------
          ivqr = irlp + iparm(3)*((mod((is),ns+1)))
          iva = ip + iparm(3)*((mod((is+1),ns+1)))
         ireq = 3
        iretlb = 2027
        go to 1500
 2027   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5602
                 go to 5603
 5602  continue
          call dwset (fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),fwk(ip  +ip
     &arm(3)*((mod((is+1),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5604
 5603 continue
          iva = ip   + iparm(3)*((mod((is+1),ns+1)))
          ivql = ilp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 5
        iretlb = 2028
        go to 1500
 2028   continue
         if (ijob .eq. -1) go to 900
 5604 continue
*        *---end revcom precon----------
                go to 5601
 5600 continue
*        *---begin revcom matvec----------
          ivqr = irlp + iparm(3)*((mod((is),ns+1)))
          iva = iv1  
         ireq = 3
        iretlb = 2029
        go to 1500
 2029   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5605
                 go to 5606
 5605  continue
          call dwset (fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),fwk(iv1 ),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5607
 5606 continue
          iva = iv1  
          ivql = ilp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 5
        iretlb = 2030
        go to 1500
 2030   continue
         if (ijob .eq. -1) go to 900
 5607 continue
*        *---end revcom precon----------
 5601 continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5608
                 go to 5609
 5608  continue
          call dwset (fwk(irlp+iparm(3)*((mod((is+1),ns+1)))),fwk(ilp+ip
     &arm(3)*((mod((is+1),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5610
 5609 continue
          ivql = ilp + iparm(3)*((mod((is+1),ns+1)))
          ivqr = irlp + iparm(3)*((mod((is+1),ns+1)))
         ireq = 7
        iretlb = 2031
        go to 1500
 2031   continue
         if (ijob .eq. -1) go to 900
 5610 continue
*        *---end revcom precon----------
 510    continue
*
        fwk(ipf    + mod((is+1),ns+1)) = fwk(ipf    + mod((is),ns+1))
        do 5611 i = 0, is
          call dwdot (pap, fwk(ilp+iparm(3)*((mod((i),ns+1)))),
     &    fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),  iparm , fparm , iwk ,
     & fwk , ier  )
          pap = pap * (fwk(ipf    + mod((i),ns+1))) * fwk(ipf    + mod((
     &is+1),ns+1))
          fwk(iu     + ((i+1)-1) + (ns+1)*((is+1)-1)) = pap
          if (iparm(2) .ge. 5) then
        write (iparm(1),'(1x,a,i5,a,i5,a,1pg20.10)') 'h(',i+1,',',is+1,'
     &)      ', pap
          endif
          val = - pap * fwk(ipf    + mod((i),ns+1)) / fwk(ipf    + mod((
     &is+1),ns+1))
          call dwsax (fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),fwk(ilp+ipa
     &rm(3)*((mod((is+1),ns+1)))),
     &    fwk(ilp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iwk
     & , fwk , ier  )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(1
     &5)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dwsax (fwk(ip+iparm(3)*((mod((is+1),ns+1)))),fwk(ip+iparm
     &(3)*((mod((is+1),ns+1)))),
     &    fwk(ip+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iwk 
     &, fwk , ier  )
          endif
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call dwsax (fwk(irlp+iparm(3)*((mod((is+1),ns+1)))),fwk(irlp+i
     &parm(3)*((mod((is+1),ns+1)))),
     &    fwk(irlp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iw
     &k , fwk , ier  )
          endif
5611  continue
*
          call dwrdot (vdot,fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),
     &    fwk(ilp+iparm(3)*((mod((is+1),ns+1)))),  iparm , fparm , iwk ,
     & fwk , ier  )
        vdot = vdot * (fwk(ipf    + mod((is+1),ns+1))) * fwk(ipf    + mo
     &d((is+1),ns+1))
        vnorm = sqrt(vdot)
        fwk(iu     + ((is+2)-1) + (ns+1)*((is+1)-1)) = vnorm
        if (iparm(2) .ge. 5) then
        write (iparm(1),'(1x,a,i5,a,i5,a,1pg20.10)') 'h(',is+2,',',is+1,
     &')      ', vnorm
        endif
        if (vnorm .eq. 0d0) then
          fwk(ipf    + mod((is+1),ns+1)) = 1d0
        else
          fwk(ipf    + mod((is+1),ns+1)) = fwk(ipf    + mod((is+1),ns+1)
     &)/vnorm
        endif
        if (.true.) then
          call dwscl ( fwk(ilp+iparm(3)*((mod((is+1),ns+1)))), fwk(ilp+i
     &parm(3)*((mod((is+1),ns+1)))), fwk(ipf    + mod((is+1),ns+1)),  ip
     &arm , fparm , iwk , fwk , ier  )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(1
     &5)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dwscl ( fwk(ip+iparm(3)*((mod((is+1),ns+1)))), fwk(ip+ipa
     &rm(3)*((mod((is+1),ns+1)))), fwk(ipf    + mod((is+1),ns+1)),  ipar
     &m , fparm , iwk , fwk , ier  )
          endif
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call dwscl ( fwk(irlp+iparm(3)*((mod((is+1),ns+1)))), fwk(irlp
     &+iparm(3)*((mod((is+1),ns+1)))), fwk(ipf    + mod((is+1),ns+1)),  
     &iparm , fparm , iwk , fwk , ier  )
          endif
          fwk(ipf    + mod((is+1),ns+1)) = 1d0
        endif
*
        do 5612 i = 1, is
          ut    = fwk(iu     + ((i  )-1) + (ns+1)*((is+1)-1))
          h     = fwk(iu     + ((i+1)-1) + (ns+1)*((is+1)-1))
          fwk(iu     + ((i  )-1) + (ns+1)*((is+1)-1)) = (fwk(icos   + mo
     &d((i),ns)))*ut + (fwk(isin   + mod((i),ns)))*h
          fwk(iu     + ((i+1)-1) + (ns+1)*((is+1)-1)) =      -fwk(isin  
     & + mod((i),ns)) *ut +       fwk(icos   + mod((i),ns)) *h
5612  continue
        v1 = fwk(iu     + ((is+1)-1) + (ns+1)*((is+1)-1))
        v2 = fwk(iu     + ((is+2)-1) + (ns+1)*((is+1)-1))
        denom = sqrt ((v1)*v1 + (v2)*v2)
        if (denom .eq. 0d0) then
                ier = -6
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Iterative method has apparently converged
     &' )
                go to 900
        endif
        fwk(icos   + mod((is+1),ns)) = v1/denom
        fwk(isin   + mod((is+1),ns)) = v2/denom
        fwk(iu     + ((is+1)-1) + (ns+1)*((is+1)-1)) = denom
        fwk(iu     + ((is+2)-1) + (ns+1)*((is+1)-1)) = 0d0
        if (iparm(2) .ge. 5) then
          do 5613 i = 1, is+1
        write (iparm(1),'(1x,a,i5,a,i5,a,1pg20.10)') 'u(',i,',',is+1,') 
     &     ', fwk(iu     + ((i)-1) + (ns+1)*((is+1)-1))
5613  continue
        endif
        fwk(izc    + mod((is+2)-1,ns+1)) =       -fwk(isin   + mod((is+1
     &),ns)) *fwk(izc    + mod((is+1)-1,ns+1))
        fwk(izc    + mod((is+1)-1,ns+1)) =  (fwk(icos   + mod((is+1),ns)
     &))*fwk(izc    + mod((is+1)-1,ns+1))
*
        nm = is + 1
        if (((mod(iparm(15)/1,2).eq.1)     .or.
     &    iparm(12).eq.-2    .or.
     &    iparm(12).eq.1     .or. iparm(12).eq.5) .or.
     &      ((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6) .or. ((mod(iparm(15)/4
     &,2).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7) .or. ((mod(iparm(15)/8,
     &2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8) .or. (is.eq.ns-1)) then
       do 5614 i = 1, nm-1
       fwk(iyold-1+i) = fwk(iy-1+i)
 5614  continue
          fwk(iyold-1+nm) = 0d0
          do 5615 i = nm, 1, -1
            sum = fwk(izc    + mod((i)-1,ns+1))
            do 5616 j = i+1, nm
              sum = sum - fwk(iy-1+j)*fwk(iu     + ((i)-1) + (ns+1)*((j)
     &-1))
5616  continue
            fwk(iy-1+i) = sum/fwk(iu     + ((i)-1) + (ns+1)*((i)-1))
5615  continue
          if (iparm(2) .ge. 5) then
            do 5617 i = 1, nm
        write (iparm(1),'(1x,a,i5,a,1pg20.10)') 'u-from-p(',i,')     ', 
     &fwk(iy-1+i)
5617  continue
          endif
        endif
*
        if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6) .or. ((mod(iparm(15)/4
     &,2).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7) .or. ((mod(iparm(15)/8,
     &2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8) .or. (is.eq.ns-1)) then
       do 5618 i = 0, nm-1
       fwk(ipzold+i) = fwk(ipz+i)
 5618  continue
          fwk(ipzold+nm) = 0d0
       do 5619 i = 1, is+1
       fwk(ipz-1+i) = fwk(izc    + mod((i)-1,ns+1))
 5619  continue
          fwk(ipz+nm) = 0d0
          do 5620 i = nm, 1, -1
            v1 = fwk(icos   + mod((i),ns))*fwk(ipz+i-1) - (fwk(isin   + 
     &mod((i),ns)))*fwk(ipz+i)
            v2 = fwk(isin   + mod((i),ns))*fwk(ipz+i-1) + (fwk(icos   + 
     &mod((i),ns)))*fwk(ipz+i)
            fwk(ipz+i-1) = v1
            fwk(ipz+i)   = v2
5620  continue
          if (iparm(2) .ge. 5) then
            do 5621 i = 1, nm+1
        write (iparm(1),'(1x,a,i5,a,1pg20.10)') 'r-from-p(',i,')     ', 
     &fwk(ipz-1+i)
5621  continue
          endif
        endif
*
        if (is .eq. ns-1) then
          call dwfil (fwk(irlq+iparm(3)*((mod((os),nsout)))),0d0,
     &     iparm , fparm , iwk , fwk , ier  )
          do 5622 i = 0, nm-1
            val = fwk(iy+i) * fwk(ipf    + mod((i),ns+1))
          call dwsax (fwk(irlq+iparm(3)*((mod((os),nsout)))),fwk(irlq+ip
     &arm(3)*((mod((os),nsout)))),
     &    fwk(irlp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iw
     &k , fwk , ier  )
5622  continue
        endif
*
        if (((mod(iparm(15)/1,2).eq.1)     .or.
     &    iparm(12).eq.-2    .or.
     &    iparm(12).eq.1     .or. iparm(12).eq.5)) then
          do 5623 i = 0, nm-1
            val = (fwk(iy+i)-fwk(iyold+i)) * fwk(ipf    + mod((i),ns+1))
          call dwsax (u,u,
     &    fwk(irlp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iw
     &k , fwk , ier  )
5623  continue
          if (is .eq. ns-1) then
          call dwsub ( u,
     &    u, fwk(irlq+iparm(3)*((mod((os),nsout)))),  iparm , fparm , iw
     &k , fwk , ier  )
          endif
        endif
*
        if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(15)
     &/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          do 5624 i = 0, is+1
            val = - (fwk(ipz+i)-fwk(ipzold+i)) * fwk(ipf    + mod((i),ns
     &+1))
          call dwsax (fwk(ir),fwk(ir),
     &    fwk(ip+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iwk 
     &, fwk , ier  )
5624  continue
          if (is .eq. ns-1) then
          call dwfil (fwk(iarlq+iparm(3)*((mod((os),nsout)))),0d0,
     &     iparm , fparm , iwk , fwk , ier  )
            do 5625 i = 0, is+1
              val = + fwk(ipz+i) * fwk(ipf    + mod((i),ns+1))
          call dwsax (fwk(iarlq+iparm(3)*((mod((os),nsout)))),fwk(iarlq+
     &iparm(3)*((mod((os),nsout)))),
     &    fwk(ip+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iwk 
     &, fwk , ier  )
5625  continue
          call dwadd (fwk(ir),
     &    fwk(ir), fwk(iarlq+iparm(3)*((mod((os),nsout)))),  iparm , fpa
     &rm , iwk , fwk , ier  )
          endif
        endif
*
        if (is .eq. ns-1) then
          call dwfil (fwk(ilarlq+iparm(3)*((mod((os),nsout)))),0d0,
     &     iparm , fparm , iwk , fwk , ier  )
          do 5626 i = 0, is+1
            val = + fwk(ipz+i) * fwk(ipf    + mod((i),ns+1))
          call dwsax (fwk(ilarlq+iparm(3)*((mod((os),nsout)))),fwk(ilarl
     &q+iparm(3)*((mod((os),nsout)))),
     &    fwk(ilp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iwk
     & , fwk , ier  )
5626  continue
        endif
*
        if (((mod(iparm(15)/4,2).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7) .or. ((.not.((iparm(13)
     &.eq.1).or.(iparm(13).eq.3))).and.((mod(iparm(15)/2,2).eq.1)     .o
     &r.
     &    iparm(12).eq.2     .or. iparm(12).eq.6))
     &              .or. ((.not.((iparm(13).eq.2).or.(iparm(13).eq.3))).
     &and.((mod(iparm(15)/8,2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8))) then
          do 5627 i = 0, is+1
            val = - (fwk(ipz+i)-fwk(ipzold+i)) * fwk(ipf    + mod((i),ns
     &+1))
          call dwsax (fwk(ilr),fwk(ilr),
     &    fwk(ilp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iwk
     & , fwk , ier  )
5627  continue
          if (is .eq. ns-1) then
          call dwadd (fwk(ilr),
     &    fwk(ilr), fwk(ilarlq+iparm(3)*((mod((os),nsout)))),  iparm , f
     &parm , iwk , fwk , ier  )
          endif
        endif
*
        if (((iparm(13).eq.2).or.(iparm(13).eq.3)) .and. is.eq.ns-1) the
     &n
          call dwfil (fwk(irarlq+iparm(3)*((mod((os),nsout)))),0d0,
     &     iparm , fparm , iwk , fwk , ier  )
          do 5628 i = 0, is+1
            val = + fwk(ipz+i) * fwk(ipf    + mod((i),ns+1))
          call dwsax (fwk(irarlq+iparm(3)*((mod((os),nsout)))),fwk(irarl
     &q+iparm(3)*((mod((os),nsout)))),
     &    fwk(irlp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iw
     &k , fwk , ier  )
5628  continue
        endif
*
        if (((iparm(13).eq.2).or.(iparm(13).eq.3)) .and. ((mod(iparm(15)
     &/8,2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8)) then
          do 5629 i = 0, is+1
            val = - (fwk(ipz+i)-fwk(ipzold+i)) * fwk(ipf    + mod((i),ns
     &+1))
          call dwsax (fwk(irlr),fwk(irlr),
     &    fwk(irlp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iw
     &k , fwk , ier  )
5629  continue
          if (is .eq. ns-1) then
          call dwadd (fwk(irlr),
     &    fwk(irlr), fwk(irarlq+iparm(3)*((mod((os),nsout)))),  iparm , 
     &fparm , iwk , fwk , ier  )
          endif
        endif
*
        if (is .eq. ns-1) then
          do 5630 i = 0, os-1
          call dwdot (vdot, fwk(ilarlq+iparm(3)*((mod((i),nsout)))),
     &    fwk(ilarlq+iparm(3)*((mod((os),nsout)))),  iparm , fparm , iwk
     & , fwk , ier  )
          call dwsax (fwk(irlq+iparm(3)*((mod((os),nsout)))),fwk(irlq+ip
     &arm(3)*((mod((os),nsout)))),
     &    fwk(irlq+iparm(3)*((mod((i),nsout)))), -vdot,  iparm , fparm ,
     & iwk , fwk , ier  )
            if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm
     &(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dwsax (fwk(iarlq+iparm(3)*((mod((os),nsout)))),fwk(iarlq+
     &iparm(3)*((mod((os),nsout)))),
     &    fwk(iarlq+iparm(3)*((mod((i),nsout)))), -vdot,  iparm , fparm 
     &, iwk , fwk , ier  )
            endif
          call dwsax (fwk(ilarlq+iparm(3)*((mod((os),nsout)))),fwk(ilarl
     &q+iparm(3)*((mod((os),nsout)))),
     &    fwk(ilarlq+iparm(3)*((mod((i),nsout)))), -vdot,  iparm , fparm
     & , iwk , fwk , ier  )
            if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call dwsax (fwk(irarlq+iparm(3)*((mod((os),nsout)))),fwk(irarl
     &q+iparm(3)*((mod((os),nsout)))),
     &    fwk(irarlq+iparm(3)*((mod((i),nsout)))), -vdot,  iparm , fparm
     & , iwk , fwk , ier  )
            endif
5630  continue
          call dwrdot (vdot,fwk(ilarlq+iparm(3)*((mod((os),nsout)))),
     &    fwk(ilarlq+iparm(3)*((mod((os),nsout)))),  iparm , fparm , iwk
     & , fwk , ier  )
          vnorm = sqrt(abs(vdot))
          if (vnorm .eq. 0d0) then
                ier = -7
                call xersho ( ier, 'dgmrrr' , iparm , ' ' )
                go to 900
          endif
          call dwscl ( fwk(irlq+iparm(3)*((mod((os),nsout)))), fwk(irlq+
     &iparm(3)*((mod((os),nsout)))), 1d0/vnorm,  iparm , fparm , iwk , f
     &wk , ier  )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(1
     &5)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dwscl ( fwk(iarlq+iparm(3)*((mod((os),nsout)))), fwk(iarl
     &q+iparm(3)*((mod((os),nsout)))), 1d0/vnorm,  iparm , fparm , iwk ,
     & fwk , ier  )
          endif
          call dwscl ( fwk(ilarlq+iparm(3)*((mod((os),nsout)))), fwk(ila
     &rlq+iparm(3)*((mod((os),nsout)))), 1d0/vnorm,  iparm , fparm , iwk
     & , fwk , ier  )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call dwscl ( fwk(irarlq+iparm(3)*((mod((os),nsout)))), fwk(ira
     &rlq+iparm(3)*((mod((os),nsout)))), 1d0/vnorm,  iparm , fparm , iwk
     & , fwk , ier  )
          endif
          call dwdot (vdot, fwk(ilarlq+iparm(3)*((mod((os),nsout)))),
     &    fwk(ilr),  iparm , fparm , iwk , fwk , ier  )
          call dwsax (u,u,
     &    fwk(irlq+iparm(3)*((mod((os),nsout)))), +vdot,  iparm , fparm 
     &, iwk , fwk , ier  )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(1
     &5)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
          call dwsax (fwk(ir),fwk(ir),
     &    fwk(iarlq+iparm(3)*((mod((os),nsout)))), -vdot,  iparm , fparm
     & , iwk , fwk , ier  )
          endif
          call dwsax (fwk(ilr),fwk(ilr),
     &    fwk(ilarlq+iparm(3)*((mod((os),nsout)))), -vdot,  iparm , fpar
     &m , iwk , fwk , ier  )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call dwsax (fwk(irlr),fwk(irlr),
     &    fwk(irarlq+iparm(3)*((mod((os),nsout)))), -vdot,  iparm , fpar
     &m , iwk , fwk , ier  )
          endif
        endif
*
        rdot  = (fwk(izc    + mod((is+2)-1,ns+1)))*fwk(izc    + mod((is+
     &2)-1,ns+1))
        if (is .eq. ns-1) then
          call dwrdot (rdot,fwk(ilr),
     &    fwk(ilr),  iparm , fparm , iwk , fwk , ier  )
        endif
        rnorm = sqrt(rdot)
*
        in         = in + 1
        iparm(5) = in
        is         = is + 1
        if (is .eq. ns) is = 0
        if (is .eq. 0) then
          os = os + 1
          if (os .eq. nsout) os = 0
        endif
*
      go to 100
*
 2500 continue
      if (iparm(2) .ge. 3) then
        write (iparm(1),'(/a,i7,a/)') '     Restarted GMRESR Method has 
     &converged in ', iparm(5), ' iterations.'
      endif
*
*-------------------------------terminate-------------------------------
*
 900  continue
*
      if (is.ne.0 .and. .not.((mod(iparm(15)/1,2).eq.1)     .or.
     &    iparm(12).eq.-2    .or.
     &    iparm(12).eq.1     .or. iparm(12).eq.5)) then
        nm = is
        do 5631 i = nm, 1, -1
          sum = fwk(izc    + mod((i)-1,ns+1))
          do 5632 j = i+1, nm
            sum = sum - fwk(iy-1+j)*fwk(iu     + ((i)-1) + (ns+1)*((j)-1
     &))
5632  continue
          fwk(iy-1+i) = sum/fwk(iu     + ((i)-1) + (ns+1)*((i)-1))
5631  continue
        if (iparm(2) .ge. 5) then
          do 5633 i = 1, nm
        write (iparm(1),'(1x,a,i5,a,1pg20.10)') 'u-from-p(',i,')     ', 
     &fwk(iy-1+i)
5633  continue
        endif
        do 5634 i = 0, nm-1
          val = fwk(iy+i) * fwk(ipf    + mod((i),ns+1))
          call dwsax (u,u,
     &    fwk(irlp+iparm(3)*((mod((i),ns+1)))), val,  iparm , fparm , iw
     &k , fwk , ier  )
5634  continue
      endif
*
      if (iclstg .ne. 1) then
        call dckstg (-1, iwkstg,  iparm , fparm , iwk , fwk , ier )
      endif
*
*     *---begin iteration statistics calculation----------
      if (iparm(21).eq.1 .and. ier.ge.0) go to 5635
                 go to 5636
 5635  continue
        call dfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
        call dfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
        call dfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
        call dfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
        istab = istab2
        if (iparm(19) .eq. 1) then
          call dwdot (fwk(istab+iud), uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
        endif
          call dwdot (fwk(istab+ibd ), b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5638
                 go to 5639
 5638  continue
          call dwset (fwk(ib2),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5640
 5639 continue
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          iva = ivitmp
          call dwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = ib2 
         ireq = 5
        iretlb = 2032
        go to 1500
 2032   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
         if (ijob .eq. -1) go to 900
 5640 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+ilbd), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call dwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5641
                 go to 5642
 5641  continue
          call dwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5643
 5642 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2033
        go to 1500
 2033   continue
         if (ijob .eq. -1) go to 900
 5643 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+irlbd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(19) .eq. 1) then
          call dwsub ( fwk(iu1),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call dwdot (fwk(istab+ied), fwk(iu1),
     &    fwk(iu1),  iparm , fparm , iwk , fwk , ier  )
        endif
*        *---begin revcom matvec----------
          call dfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          ivqr = ivitmp
          call dwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ib1 
         ireq = 3
        iretlb = 2034
        go to 1500
 2034   continue
          call dffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call dwsub ( fwk(ib1),
     &    b, fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
          call dwdot (fwk(istab+ird), fwk(ib1),
     &    fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5644
                 go to 5645
 5644  continue
          call dwset (fwk(ib2),fwk(ib1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5646
 5645 continue
          iva = ib1 
          ivql = ib2 
         ireq = 5
        iretlb = 2035
        go to 1500
 2035   continue
         if (ijob .eq. -1) go to 900
 5646 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+ilrd ), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call dwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5647
                 go to 5648
 5647  continue
          call dwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5649
 5648 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2036
        go to 1500
 2036   continue
         if (ijob .eq. -1) go to 900
 5649 continue
*        *---end revcom precon----------
          call dwdot (fwk(istab+irlrd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(19) .eq. 1) then
          if (fwk(istab1+ied) .eq. 0d0) then
                ier = 4
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Value of RELERR may be inaccurate' )
          else
            fparm(8) = sqrt(fwk(istab2+ied)/fwk(istab1+ied))
          endif
        endif
        if (fwk(istab1+ird) .eq. 0d0) then
                ier = 4
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Value of RELRSD may be inaccurate' )
        else
          fparm(7) = sqrt(fwk(istab2+ird)/fwk(istab1+ird))
        endif
        if (iparm(2) .ge. 4) then
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        write (iparm(1),'(5x,''Final iteration statistics'')') 
        write (iparm(1),'()') 
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10)'
     &) sqrt(fwk(istab+iud))
        endif
        write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10)'
     &) sqrt(fwk(istab+ibd))
        write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10)'
     &) sqrt(fwk(istab+ilbd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10)'
     &) sqrt(fwk(istab+irlbd))
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of error             '',1pg20.10)'
     &) sqrt(fwk(istab+ied))
        endif
        write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10)'
     &) sqrt(fwk(istab+ird))
        write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10)'
     &) sqrt(fwk(istab+ilrd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10)'
     &) sqrt(fwk(istab+irlrd))
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''Relative 2-norm of error    '',1pg20.10)'
     &) fparm(8)
        endif
        write (iparm(1),'(5x,''Relative 2-norm of residual '',1pg20.10)'
     &) fparm(7)
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        endif
        call dffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call dffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call dffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call dffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call dffre (istab2,8, iparm , fparm , iwk , fwk , ier )
        call dffre (istab1,8, iparm , fparm , iwk , fwk , ier )
                go to 5637
 5636 continue
 5637 continue
*     *---end iteration statistics calculation----------
*
      call dffre (ipzold,ns+1, iparm , fparm , iwk , fwk , ier )
      call dffre (ipz   ,ns+1, iparm , fparm , iwk , fwk , ier )
      call dffre (iyold ,ns, iparm , fparm , iwk , fwk , ier )
      call dffre (iy    ,ns, iparm , fparm , iwk , fwk , ier )
      call dffre (izc   ,ns+1, iparm , fparm , iwk , fwk , ier )
      call dffre (iu    ,ns*(ns+1), iparm , fparm , iwk , fwk , ier )
      call dffre (isin  ,ns+1, iparm , fparm , iwk , fwk , ier )
      call dffre (icos  ,ns+1, iparm , fparm , iwk , fwk , ier )
      call dffre (ipf   ,ns+1, iparm , fparm , iwk , fwk , ier )
      call dffre (iv2 ,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call dffre (iv1 ,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        call dffre (irarlq,iparm(3)*(ns), iparm , fparm , iwk , fwk , ie
     &r )
        call dffre (irlp,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ie
     &r )
        call dffre (irlr,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      endif
      if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(15)/2
     &,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6)) then
        call dffre (iarlq,iparm(3)*(ns), iparm , fparm , iwk , fwk , ier
     & )
        call dffre (ip,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier 
     &)
        call dffre (ir,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      endif
      call dffre (ilarlq,iparm(3)*(ns), iparm , fparm , iwk , fwk , ier 
     &)
      call dffre (irlq,iparm(3)*(ns), iparm , fparm , iwk , fwk , ier )
      call dffre (ilp,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier )
      call dffre (ilr,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
*
*
*
*
      call dffre (iftab,5, iparm , fparm , iwk , fwk , ier )
      call xifre (iitab,46, iparm , fparm , iwk , fwk , ier )
*
      if (itimer .eq. 1) then
        call xtimer (tc2,tr2,iparm(22),0)
        fparm(1) = fparm(1) + (tc2-tc1)
        fparm(2) = fparm(2) + (tr2-tr1)
      endif
*
 910  continue
      if (iparm(27) .eq. 0) then
        call difppr ( -1 , iparm, fparm, ier )
      endif
*
*     *---reset iclev----------
      if (iparm(27) .eq. 0) iparm(27) = 0
      iparm(11) = (iparm(6))
      ireq        = -1
      return
*
*-------------------------------brancher--------------------------------
*
*
 1000 continue
*
      ipme  = 0
      nproc = 1
      iitab = iparm(11)
      if (iitab.eq.(iparm(6)) .and. ijob.eq.-1) go to 910
*
      if (iparm(11) .eq. (iparm(6))) then
                ier = -4
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Values of ijob and iparm(iptr) incompatib
     &le' )
      endif
*
      call xtimer (tc1,tr1,iparm(22),0)
*
*     *---remember parameters out of tables----------
        iitab  = iwk(iitab+0)
        iftab  = iwk(iitab+1)
        istab  = iwk(iitab+2)
        iwkstg = iwk(iitab+3)
        iclstg = iwk(iitab+4)
        in     = iwk(iitab+5)
        is     = iwk(iitab+6)
        itimer = iwk(iitab+7)
        iretlb = iwk(iitab+8)
        ivitmp = iwk(iitab+9)
        ivotmp = iwk(iitab+10)
        iv1tst = iwk(iitab+11)
        iv2tst = iwk(iitab+12)
        istab1 = iwk(iitab+13)
        istab2 = iwk(iitab+14)
        ib1    = iwk(iitab+15)
        ib2    = iwk(iitab+16)
        iu1    = iwk(iitab+17)
        iu2    = iwk(iitab+18)
        ir     = iwk(iitab+19)
        ilr    = iwk(iitab+20)
        irlr   = iwk(iitab+21)
        ip     = iwk(iitab+22)
        ilp    = iwk(iitab+23)
        irlp   = iwk(iitab+24)
        irlq   = iwk(iitab+25)
        iarlq  = iwk(iitab+26)
        ilarlq = iwk(iitab+27)
        irarlq = iwk(iitab+28)
        iv1    = iwk(iitab+29)
        iv2    = iwk(iitab+30)
        ipf    = iwk(iitab+31)
        icos   = iwk(iitab+32)
        isin   = iwk(iitab+33)
        iu     = iwk(iitab+34)
        izc    = iwk(iitab+35)
        iy     = iwk(iitab+36)
        iyold  = iwk(iitab+37)
        ipz    = iwk(iitab+38)
        ipzold = iwk(iitab+39)
        ns     = iwk(iitab+40)
        os     = iwk(iitab+41)
        nsout  = iwk(iitab+42)
        i      = iwk(iitab+43)
        i      = iwk(iitab+44)
        nm = iwk(iitab+45)
        top = fwk(iftab+0)
        bot = fwk(iftab+1)
        rdot   = fwk(iftab+2)
        rnorm  = fwk(iftab+3)
        r0norm = fwk(iftab+4)
*
      if (ijob.ne.3 .and. ijob.ne.-1) then
                ier = -4
                call xersho ( ier, 'dgmrrr' , iparm ,
     &                        'Argument ijob invalid' )
      endif
*
*     *---perform the branch to where we left off----------
      go to (2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,2011
     &,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024,
     &2025,2026,2027,2028,2029,2030,2031,2032,2033,2034,2035,2036),
     &       iretlb - 2000 + 1
*
*---------------------------request of caller---------------------------
*
*
 1500 continue
      iparm(11) = iitab
*     *---save tables----------
        iwk(iitab+0) = iitab 
        iwk(iitab+1) = iftab 
        iwk(iitab+2) = istab 
        iwk(iitab+3) = iwkstg
        iwk(iitab+4) = iclstg
        iwk(iitab+5) = in    
        iwk(iitab+6) = is    
        iwk(iitab+7) = itimer
        iwk(iitab+8) = iretlb
        iwk(iitab+9) = ivitmp
        iwk(iitab+10) = ivotmp
        iwk(iitab+11) = iv1tst
        iwk(iitab+12) = iv2tst
        iwk(iitab+13) = istab1
        iwk(iitab+14) = istab2
        iwk(iitab+15) = ib1   
        iwk(iitab+16) = ib2   
        iwk(iitab+17) = iu1   
        iwk(iitab+18) = iu2   
        iwk(iitab+19) = ir    
        iwk(iitab+20) = ilr   
        iwk(iitab+21) = irlr  
        iwk(iitab+22) = ip    
        iwk(iitab+23) = ilp   
        iwk(iitab+24) = irlp  
        iwk(iitab+25) = irlq  
        iwk(iitab+26) = iarlq 
        iwk(iitab+27) = ilarlq
        iwk(iitab+28) = irarlq
        iwk(iitab+29) = iv1   
        iwk(iitab+30) = iv2   
        iwk(iitab+31) = ipf   
        iwk(iitab+32) = icos  
        iwk(iitab+33) = isin  
        iwk(iitab+34) = iu    
        iwk(iitab+35) = izc   
        iwk(iitab+36) = iy    
        iwk(iitab+37) = iyold 
        iwk(iitab+38) = ipz   
        iwk(iitab+39) = ipzold
        iwk(iitab+40) = ns    
        iwk(iitab+41) = os    
        iwk(iitab+42) = nsout 
        iwk(iitab+43) = i     
        iwk(iitab+44) = i     
        iwk(iitab+45) = nm
        fwk(iftab+0) = top
        fwk(iftab+1) = bot
        fwk(iftab+2) = rdot  
        fwk(iftab+3) = rnorm 
        fwk(iftab+4) = r0norm
      call xtimer (tc2,tr2,iparm(22),0)
      fparm(1) = fparm(1) + (tc2-tc1)
      fparm(2) = fparm(2) + (tr2-tr1)
      return
*
*
*--------------------------------end------------------------------------
*
  500 format ( 1x, a, 1pg20.10 )
*
      end
